use std::{io, sync::Arc, thread::yield_now};

use tokio::{sync::RwLock, task::JoinHandle};

use super::atem_socket_inner::AtemSocketInner;

pub struct AtemSocket {
    socket: Arc<RwLock<AtemSocketInner>>,

    inner_socket_handle: JoinHandle<()>,
}

#[derive(Debug, Error)]
pub enum AtemSocketConnectionError {
    #[error("Socket connection error")]
    IoError(#[from] io::Error),
}

impl AtemSocket {
    pub fn new() -> Self {
        let socket = AtemSocketInner::new();
        let socket = Arc::new(RwLock::new(socket));

        let socket_clone = Arc::clone(&socket);
        let handle = tokio::spawn(async move {
            loop {
                socket_clone.write().await.tick().await;

                yield_now();
            }
        });

        AtemSocket {
            socket,

            inner_socket_handle: handle,
        }
    }

    pub async fn connect(
        &mut self,
        address: String,
        port: u16,
    ) -> Result<(), AtemSocketConnectionError> {
        self.socket.write().await.connect(address, port).await?;

        Ok(())
    }

    pub async fn disconnect(self) {
        self.inner_socket_handle.abort();
        self.socket.write().await.disconnect()
    }

    pub async fn send_command(&mut self, payload: &[u8], raw_name: &str, tracking_id: u64) {
        self.socket
            .write()
            .await
            .send_command(payload, raw_name, tracking_id)
            .await;
    }
}

impl Default for AtemSocket {
    fn default() -> Self {
        Self::new()
    }
}
