use std::collections::HashMap;

use crate::{enums::ProtocolVersion, state::AtemState};

pub trait IDeserializedCommand {
    fn apply_to_state(&self, state: &mut AtemState) -> Vec<String>;
}

pub trait DeserializedCommand: IDeserializedCommand {
    fn get_raw_name(&self) -> Option<String>;
    fn get_minimum_version(&self) -> Option<ProtocolVersion>;
}

pub trait ISerializableCommand {
    fn serialize(version: ProtocolVersion) -> Vec<u8>;
}

pub trait BasicWritableCommand: ISerializableCommand {
    fn get_raw_name(&self) -> Option<String>;
    fn get_minimum_version(&self) -> Option<ProtocolVersion>;
}

pub trait WritableCommand: BasicWritableCommand {
    fn get_mask_flag(&self) -> HashMap<String, f64>;
    fn get_flag(&self) -> f64;
    fn set_flag(&mut self, flag: f64);
}

pub trait SymmetricalCommand: DeserializedCommand + ISerializableCommand {}
