use crate::enums::{Model, ProtocolVersion};

#[derive(Getters, new)]
pub struct AtemCapabilites {
    mix_effects: u64,
    sources: u64,
    auxilliaries: u64,
    mix_minus_outputs: u64,
    media_players: u64,
    serial_ports: u64,
    max_hyperdecks: u64,
    dves: u64,
    stingers: u64,
    super_sources: u64,
    talkback_channels: u64,
    downstream_keyers: u64,
    camera_control: bool,
    advanced_chroma_keyers: bool,
    only_configurable_outputs: bool,
}

#[derive(Getters, new)]
pub struct MixEffectInfo {
    key_count: u64,
}

#[derive(Getters, new)]
pub struct SuperSourceInfo {
    box_count: u64,
}

#[derive(Getters, new)]
pub struct AudioMixerInfo {
    inputs: u64,
    monitors: u64,
    headphones: u64,
}

#[derive(Getters, new)]
pub struct FairlightAudioMixerInfo {
    inputs: u64,
    monitors: u64,
}

#[derive(Getters, new)]
pub struct MacroPoolInfo {
    macro_count: u64,
}

#[derive(Getters, new)]
pub struct MediaPoolInfo {
    still_count: u64,
    clip_count: u64,
}

#[derive(Getters, new)]
pub struct MultiviewerInfo {
    count: u64,
    window_count: u64,
}

#[derive(new)]
pub struct TimeInfo {
    pub hour: u64,
    pub minute: u64,
    pub second: u64,
    pub frame: u64,
    pub drop_frame: bool,
}

#[derive(new, Default)]
pub struct DeviceInfo {
    pub api_version: ProtocolVersion,
    pub capabilities: Option<AtemCapabilites>,
    pub model: Model,
    pub product_identifier: Option<String>,
    pub super_sources: Vec<Option<SuperSourceInfo>>,
    pub mix_effects: Vec<Option<MixEffectInfo>>,
    pub power: Vec<bool>,
    pub audio_mixer: Option<AudioMixerInfo>,
    pub fairlight_mixer: Option<FairlightAudioMixerInfo>,
    pub macro_pool: Option<MacroPoolInfo>,
    pub media_pool: Option<MediaPoolInfo>,
    pub multiviewer: Option<MultiviewerInfo>,
    pub lastTime: Option<TimeInfo>,
}
