use crate::enums::{MultiViewerLayout, VideoMode};

pub trait MultiViewerSourceState {
    fn get_source(&self) -> u64;
    fn set_source(&mut self, source: u64);

    fn get_window_index(&self) -> u64;
    fn get_supports_vu_meter(&self) -> bool;
    fn get_supports_safe_area(&self) -> bool;
}

#[derive(Getters, new)]
pub struct MultiViewerWindowState {
    pub safe_title: Option<bool>,
    pub audio_meter: Option<bool>,

    // SourceState props
    pub source: u64,
    window_index: u64,
    supports_vu_meter: bool,
    supports_safe_area: bool,
}

impl MultiViewerSourceState for MultiViewerWindowState {
    fn get_source(&self) -> u64 {
        self.source
    }
    fn set_source(&mut self, source: u64) {
        self.source = source;
    }

    fn get_window_index(&self) -> u64 {
        self.window_index
    }
    fn get_supports_vu_meter(&self) -> bool {
        self.supports_vu_meter
    }
    fn get_supports_safe_area(&self) -> bool {
        self.supports_safe_area
    }
}

#[derive(Getters, new)]
pub struct MultiViewerPropertiesState {
    pub layout: MultiViewerLayout,
    pub program_preview_swapped: bool,
}

#[derive(Getters, new)]
pub struct MultiViewer {
    index: u64,
    windows: Vec<MultiViewerWindowState>,
    pub properties: Option<MultiViewerPropertiesState>,
    pub vu_opacity: Option<u64>,
}

#[derive(Getters, new, Default)]
pub struct SettingsState {
    multi_viewers: Vec<MultiViewer>,
    pub video_mode: VideoMode,
}
